// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include <orbitersdk.h>
#include <cstdio>
#include <cmath>
#include <iostream>

#include <string>
#include "OrbiterSoundSDK3.h"

#include <stdio.h>
#include <stdlib.h>

using namespace std;


// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 10000.0;
const double PB_ISP = 50000.0;
const double PB_MAXMAINTH =0;

double Cockpit= 0;
double iop = 0;
double st = 0;
double pld = 0;
double cen = 0;
double gear_proc = 0;
double gear_procl = 0;
double exproc = 0;
double Lights = 0;
double switched = 0;
double aoa;
double dil = 0;
double eng_pitch = 12000;

double start = 0;
double full = 0.001;
double i = 0;
double i1 = 0;
double i2 = 0;
double i3 = 0;
double i4 = 0;
double i5 = 0;
double i7 = 0;
double i9 = 0;
double i1a = 0;
double i2a = 0;
double i3a = 0;
double i4a = 0;
double i5a = 0;

double power_proc = 0;

double i13 = 0;
double i14 = 0;
double i15 = 0;
double i16 = 0;
double i17 = 0;
double i18 = 0;
double i19 = 0;
double i20 = 0;
double i21 = 0;

double i25 = 0;
double i26 = 0;


double i33 = 0;

double s1 = 0;
double s2 = 0;
double s3 = 0;
double s4 = 0;
double s5 = 0;
double ga = 0;
double diff = 0.1;
double k3;
int k4;
char k3buffer [33];
double amet = 0;
double dee = 0;
double dee33 = 0;
double propanm = 0;
double propanm2 = 0;

double refuel = 0;

double flaps = 0;

double move = 2;
double move_down = 0.333;

UINT animactive;

UINT animactivel;

UINT animactive2;

UINT animactive3;

UINT animactive4;

const VECTOR3 offset = {0,0 - move_down ,4.1 - move};

const VECTOR3 offset_prop = {0,-0.2983013,3.624378};

const VECTOR3 offset_propb = {0,-0.2983013,3.071};

const VECTOR3 offset_hor2 = {-0.4165482,0.2816617 - move_down ,4.81 - move};

const VECTOR3 offset_yaw = {-0.58,0.12 - move_down ,4.8 - move};

THRUSTER_HANDLE th_main;

BEACONLIGHTSPEC beacon1;    

BEACONLIGHTSPEC beacon2;   

BEACONLIGHTSPEC beacon3;   


MESHHANDLE f150;
MESHHANDLE f151;
MESHHANDLE f152;
MESHHANDLE f153;
MESHHANDLE f154;
MESHHANDLE f155;
MESHHANDLE f156;

// 1. vertical lift component (wings and body)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6);
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class ShuttlePB: public VESSEL2 {
public:
	ShuttlePB (OBJHANDLE hVessel, int flightmodel);
	//	: VESSEL2 (hVessel, flightmodel) {}
	
	~ShuttlePB();

	
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations(void);
	void Steering();
	void Lighting();
	void clbkPostCreation (void);
    void clbkPostStep (double simt, double simdt, double mjd);
    int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
   
    UINT anim_wheel;
    
   
    UINT anim_steer;
    
    UINT anim_flaps;

    UINT anim_dial44;

    
    //UINT anim_gear;
    UINT anim_gearlever;
    
    UINT anim_rudder;
    UINT anim_elevator;
    UINT anim_laileron;
    UINT anim_raileron;
    
    UINT anim_propelar;
    
    UINT anim_wheel0;
    UINT anim_wheel1;
    UINT anim_wheel2;
    UINT anim_wheel3;
    UINT anim_wheel4;
    UINT anim_wheel5;
    
    UINT anim_dial1;
    UINT anim_dial2;
    UINT anim_dial3;

    UINT anim_bank;
    UINT anim_pitch;

    UINT anim_power;

    UINT anim_clock_s;
    UINT anim_clock_m;
    UINT anim_clock_h;
    
    bool clbkLoadVC (int id);
    
    MGROUP_TRANSFORM *pitch1;

    
    int MySoundID;	


};

ShuttlePB::ShuttlePB (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel)
{
	
	// preload meshes
	
    f150                 = oapiLoadMeshGlobal ("K33\\K-C-172");
    f151                 = oapiLoadMeshGlobal ("K33\\K-C-172-Cpit");
    f152                 = oapiLoadMeshGlobal ("K33\\K-C-172-Prop");
    f153                 = oapiLoadMeshGlobal ("K33\\K-C-172-Prop-Blurred");
    f154                 = oapiLoadMeshGlobal ("K33\\K-C-172-Cpit_n");
    f155                 = oapiLoadMeshGlobal ("K33\\K-C-172-Cpit-Hor2");
    f156                 = oapiLoadMeshGlobal ("K33\\K-C-172-Cpit-Yaw");
	
	DefineAnimations ();
}


void ShuttlePB::clbkPostCreation (void)
{

int MySoundID=ConnectToOrbiterSoundDLL3(GetHandle());

ReplaceStockSound3(MySoundID,"Sound\\K-C-172\\K-C-172-Eng.wav",	REPLACE_MAIN_THRUST);


SoundOptionOnOff3(MySoundID,PLAYHOVERTHRUST,FALSE);


SoundOptionOnOff3(MySoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(MySoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(MySoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(MySoundID,DISPLAYTIMER,FALSE);

}


// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------

void ShuttlePB::clbkSetClassCaps (FILEHANDLE cfg)
{
	// THRUSTER_HANDLE th_main;


	// physical specs
	SetSize (5);
	SetEmptyMass (12000.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (1.7);
	SetWingEffectiveness (3.1);
	SetCrossSections (_V(5,5,15));
	SetRotDrag (_V(3.5,3.5,3.5));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(15.5,22.1,7.7));
	SetTrimScale (0.05);
	SetCameraOffset (_V(-0.4,0.555,3.3));
	
	SetTouchdownPoints (_V(0,-1.522213,2.815174), _V(-1.232543,-1.42991,1.217935), _V(1.232543,-1.42991,1.217935));

	// **************************** aerodynamics ***********************************

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.561), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);





	CreateControlSurface (AIRCTRL_ELEVATOR,     7, 0.7, _V(   0,0,-17), AIRCTRL_AXIS_XPOS,anim_elevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2, 0.5, _V(   0,2,-15), AIRCTRL_AXIS_YPOS,anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V( 17.8,0,-17.8), AIRCTRL_AXIS_XPOS,anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      4.8, 0.2, _V(-17.8,0,-17.8), AIRCTRL_AXIS_XNEG,anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 5, 0.1, _V(   0,0,-15), AIRCTRL_AXIS_XPOS,anim_elevator);
    CreateControlSurface (AIRCTRL_FLAP,         5, 5, _V( 0,0,-1), AIRCTRL_AXIS_XPOS,anim_flaps);
	

	
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (PB_FUELMASS);
	
	
	

	// ***************** thruster definitions *******************

	
	
	th_main = CreateThruster (_V(0,0,-5), _V(0,0,1), PB_MAXMAINTH, hpr, PB_ISP);
	
	
	
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
	
	
	
	// Strobe lights
	
	//Blue strobe Right Wing
	
	static VECTOR3 beacon1pos = {5.31316,0.5186034,1.567917};
	static VECTOR3 beacon1col = {0.043, 0.161, 0.992};
	
		beacon1.shape = (BEACONSHAPE_STAR);
		beacon1.pos = &beacon1pos;
		beacon1.col = &beacon1col;
		beacon1.size = (0.12);
		beacon1.falloff = (0.6);
		beacon1.period = (1.13);
		beacon1.duration = (0.05);
		beacon1.tofs = (0.6);
		beacon1.active = false;
		
		AddBeacon (&beacon1);
		
		//Red strobe Left Wing
	
	static VECTOR3 beacon2pos = {-5.31316,0.5186034,1.567917};
	static VECTOR3 beacon2col = {0.988, 0.133, 0.133};
	
		beacon2.shape = (BEACONSHAPE_STAR);
		beacon2.pos = &beacon2pos;
		beacon2.col = &beacon2col;
		beacon2.size = (0.12);
		beacon2.falloff = (0.6);
		beacon2.period = (1.13);
		beacon2.duration = (0.05);
		beacon2.tofs = (0.6);
		beacon2.active = false;
		
		AddBeacon (&beacon2);
		
//Red strobe Tail
	
	static VECTOR3 beacon3pos = {0,1.506748,-3.371929};
	static VECTOR3 beacon3col = {0.988, 0.133, 0.133};
	
		beacon3.shape = (BEACONSHAPE_STAR);
		beacon3.pos = &beacon3pos;
		beacon3.col = &beacon3col;
		beacon3.size = (0.12);
		beacon3.falloff = (0.6);
		beacon3.period = (1.13);
		beacon3.duration = (0.05);
		beacon3.tofs = (0.6);
		beacon3.active = false;
		
		AddBeacon (&beacon3);

	
	
	
	
 
	// visual specs
	
	AddMesh (f150);
    SetMeshVisibilityMode( AddMesh (f151,&offset), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f152, &offset_prop), MESHVIS_ALWAYS );
    SetMeshVisibilityMode( AddMesh (f155, &offset_hor2), MESHVIS_VC );
    SetMeshVisibilityMode( AddMesh (f156, &offset_yaw), MESHVIS_VC );

    

}

// ==============================================================
// API callback interface
// ==============================================================

void ShuttlePB::clbkPostStep (double simt, double simdt, double mjd)

{
	
	if( oapiGetTimeAcceleration() > 10.0 ) oapiSetTimeAcceleration(10);
	
	double vsAirSpeed = GetAirspeed();
	
	
	
	
    //speed and altitude control
	
	if (GetAirspeed() >= 177) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}

if (GetAltitude() >= 7000) 
	{
		SetCW (999, 999, 999, 999);
		SetThrusterMax0 (th_main ,0);

	
}



if (GetAirspeed() < 177 && GetAltitude() < 7000 ) 
	{
		SetCW (0.3, 0.3, 0.67, 0.92);
		SetThrusterMax0 (th_main ,3e5);

	
}

	double vsAirSpeed2 = (vsAirSpeed/1000)*3600;
    
    //Ground Steering
    
    if (GroundContact () ) Steering();
    
    
    //aoa = GetAOA();
    
    //sprintf(oapiDebugString(),"anim %2.2f", aoa );
    
  

    //Anims 
    
    if (iop > 1)iop=0;
    
    if (vsAirSpeed > 0) {
    
	double aspeed = vsAirSpeed;
    if (iop > 1)iop=0;
    if (vsAirSpeed > 30) aspeed = 30; 
    iop = iop + (aspeed /300);
     
    if (vsAirSpeed ==0) iop=0;
    
    
    if ( GroundContact() && gear_proc < 0.05) SetAnimation (anim_wheel, iop);
    

   }
	
  
   //Prop blurred managment
   
   if (vsAirSpeed > 1) {
	
	// SetMeshVisibilityMode( InsertMesh (f155,1,&offset), MESHVIS_VC );
     
   SetMeshVisibilityMode( InsertMesh (f153,2,&offset_propb), MESHVIS_ALWAYS );
   
}

   if (vsAirSpeed <= 1) {
	   
   SetMeshVisibilityMode( InsertMesh (f152,2,&offset_prop), MESHVIS_ALWAYS );
   
}

   
    // Flaps


   if( animactive4 ) {
		if( animactive4 == 1 ) {
			flaps += 0.006;
			if( flaps > 1.0 ) {
				flaps = 1.0;
				animactive4 = 0;
			}
		}
		else  {
			flaps -= 0.006;
			if( flaps < 0.0 ) {
				flaps = 0.0;
				animactive4 = 0;
			}
		}
		SetAnimation( anim_flaps, flaps );
	}

   SetControlSurfaceLevel(AIRCTRL_FLAP,flaps);



	
/*	
   
// Engine sound
    bool Came = oapiCameraInternal ();
	
	if (Came == true)Cockpit = 1;
	
	if (Came == false)Cockpit = 0;
	
	
	if (GetThrusterLevel (th_main) > 0 && Cockpit == 0) 
	{
		
	double pit = ((GetThrusterLevel (th_main)*6000));
	
	eng_pitch = 10000 + pit;
	
	if (eng_pitch < 10000 ) eng_pitch = 10000;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	PlayVesselWave3(MySoundID,ENG,LOOP,255,eng_pitch);
}

if (GetThrusterLevel (th_main) > 0 && Cockpit == 1) 
	{
		
	double pit = ((GetThrusterLevel (th_main)*6000));
	
	eng_pitch = 7000 + pit;
	
	if (eng_pitch < 10000 ) eng_pitch = 10000;
	
	if (eng_pitch > 22500 ) eng_pitch = 22500;
	
	//sprintf(oapiDebugString(),"anim %7.7f", eng_pitch );
		
	PlayVesselWave3(MySoundID,ENG,LOOP,225,eng_pitch);
}



    if (GetThrusterLevel (th_main) == 0) 
	{
	
	StopVesselWave3(MySoundID,ENG);
}

   
 */
   
   // Prop Animation
   
   if (GetThrusterGroupLevel (th_main) >= 0.1 )propanm2 = 1;
   
   if (GetThrusterGroupLevel (th_main) < 0.1 && GetAirspeed() <= 0.1 )propanm2 = 0;
   
  
   
   if (GetAirspeed() > 0.1 )propanm2 = 1;
   
   if (propanm2 == 1 )
   
   {
	   if (propanm > 1)propanm=0;
	   
    propanm = propanm + 0.1;
     
       
	   SetAnimation (anim_propelar ,propanm);
	   
   }
  
   
   if (GetThrusterGroupLevel (th_main) == 0 && GetAirspeed() <= 0.1)SetAnimation (anim_propelar ,0);
   
   
   if (Lights == 1 && switched == 1)
 {
	 beacon1.active = true;
	 beacon2.active = true;
	 beacon3.active = true;
	 switched = 0;
	 Lighting();
}

if (Lights == 0 && switched == 1)
{
	beacon1.active = false;
	beacon2.active = false;
	beacon3.active = false;
    switched = 0;
    Lighting();
}



   // VSI Dial Animation

    aoa = GetPitch();
    
    
   i7 = ( aoa + 1) * 0.5;
   
   if ( i7 > 1 ) i7 = 1;
   if ( i7 < 0 ) i7 = 0;
   
   //sprintf(oapiDebugString(),"vsi %5.5f", aoa );

   SetAnimation (anim_dial1, i7);
   
   
   // Direction Dial Animation

   oapiGetFocusHeading(&dil);
    
   i9 = (dil / 6.25) + 0.001;
   
   //sprintf(oapiDebugString(),"hed %2.2f", dil );

   SetAnimation (anim_dial2, i9);

    // Horizon Bank Guage Animation
    
   i15 = GetBank();
   
   i16 = (i15 + 3) / 6;
   
   if (i16 < 0) i16 = 0;
   
   if (i16 > 1) i16 = 1;
   
   SetAnimation (anim_bank, i16);
   
    //sprintf(oapiDebugString(),"hed %5.5f", i15 );
    
    
    
     // Pitch Guage Animation
    
   i25 = GetPitch();
   
   i26 = (i25 + 3) / 6;
   
   if (i26 < 0) i26 = 0;
   
   if (i26 > 1) i26 = 1;
   
   SetAnimation (anim_pitch, i26);



    // Throttle animation

        power_proc = GetThrusterLevel(th_main);
	
		SetAnimation( anim_power, power_proc );
		


   // Speed Dial Animation
    
   i13 = GetAirspeed();
   
   if (i13 > 220) i13 = 220;
   
   if (i13 < 0) i13 = 0;
   
   i14 = (-i13 + 220)/220;
   
   if (i14 < 0) i14 = 0;
   
   if (i14 > 1) i14 = 1;
   
   SetAnimation (anim_dial3, i14);


   
   
   
   // Clock Dial Animation
     
     //Second Hand
     
    
   i17 = i17 + ( 0.0002 * oapiGetTimeAcceleration() );
   
   if (i17 < 0) i17 = 0;
   
   if (i17 >= 1) 
   {
	   i17 = 0;
	   i18 = i18 + 0.016666666;
}
   SetAnimation (anim_clock_s, i17);
   
   //sprintf(oapiDebugString(),"mouse %19.19f", mx1 );
   
    //Minute Hand
    
   if (i18 < 0) i18 = 0;
   
   if (i18 >= 1) 
   {
	   i18 = 0;
	   i19 = i19 + 0.016666666;
   }
   
   
   SetAnimation (anim_clock_m, i18);
   
    //Hour Hand
   
   if (i19 < 0) i19 = 0;
   
   if (i19 >= 1) i19 = 0;
   
   SetAnimation (anim_clock_h, i19);
   
   

 // Fuel Guage Animation
   
   i20 = (-GetFuelMass() + 10000)/10000; 

   i21 = i20 / 4;
   
   if (i21 < 0) i21 = 0;
   
   if (i21 > 1) i21 = 1;
   
   SetAnimation (anim_dial44, i21);

   
   // Refuel
   
   if (refuel == 1)
   {
	   
	  SetFuelMass (10000.0);

	   refuel = 0;
   }

   
   // Altimeter animation
   
   
    //Anims 
    
     k3 = GetAltitude();
    
     k4 = int (k3);
      
     gcvt (k3,6,k3buffer);
     
     

   
     string stringOne(k3buffer);
     
     if (k3 >= 0 )
     {

     string is1 = stringOne.substr(0 + s2 + s3 + s4 + s5, 1) ;
       
     char const *cString1 = is1.c_str();
     
     i1a = atol(cString1);
     
     i1 = i1a * 0.1;
     
 }
 
 if (k3 >= 10 ) 
 {
	 
     string is2 = stringOne.substr(0 + s3 + s4 + s5 , 1) ;
       
     char const *cString2 = is2.c_str();
     
     i2a = atol(cString2);
     
     i2 = i2a * 0.1;
      s2 = 1;
 }
 
 if (k3 >= 100 )
 {
     string is3 = stringOne.substr(0 + s4 + s5, 1) ;
       
     char const *cString3 = is3.c_str();
     
     i3a = atol(cString3);
     
     i3 = i3a * 0.1;
     
     s3 = 1;
 }
 
 if (k3 >= 1000)
 {
     string is4 = stringOne.substr(0 + s5, 1) ;
       
     char const *cString4 = is4.c_str();
     
     i4a = atol(cString4);
     
     i4 = i4a * 0.1;
     
     s4 = 1;
 }
 
 if (k3 >= 10000)
 {
     string is5 = stringOne.substr(0, 1) ;
       
     char const *cString5 = is5.c_str();
     
     i5a = atol(cString5);
     
     i5 = i5a * 0.1;
     
     s5 = 1;
}

      if(k3<10)
      {
	      s2=0;
	      i2=0;
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
         
    if(k3<100)
      {
	     
	      s3=0;
	      i3=0;
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
    
    if(k3<1000)
      {
	      
	      s4=0;
	      i4=0;
	      s5=0;
	      i5=0;
    }
   
    if(k3<10000)
      {
	     
	      s5=0;
	      i5=0;
	      
    }
    
    
   i=(k3 * 0.01)- dee;
   
   
   
   if (i>=1)
   {
	   dee = dee + 1;
   }
   
   if (i <= 0)
   {
	   dee = dee - 1;
	   
	   if (dee < 0)dee = 0;
   }
   
   
    i33=(k3 * 0.001)- dee33;
   
   
   
   if (i33>=1)
   {
	   dee33 = dee33 + 1;
   }
   
   if (i33 <= 0)
   {
	   dee33 = dee33 - 1;
	   
	   if (dee33 < 0)dee33 = 0;
   }
   
    
	//sprintf(oapiDebugString(),"force %2.2f", i1 );
	
	
	SetAnimation (anim_wheel0, i);
	SetAnimation (anim_wheel1, i33);
	//SetAnimation (anim_wheel2, i2);
	//SetAnimation (anim_wheel3, i3);
	//SetAnimation (anim_wheel4, i4);
	//SetAnimation (anim_wheel5, i5);
	
	

   
   
}  
    
// Lighting


void ShuttlePB::Lighting()
 {
	 
	 
	 
    if(Lights == 1)
    {
	
	SetMeshVisibilityMode( InsertMesh (f154,1,&offset), MESHVIS_VC );
	//SetMeshVisibilityMode( InsertMesh (f156,2,&offsetalt), MESHVIS_VC );
	//SetMeshVisibilityMode( InsertMesh (f157,3,&offsetaoa), MESHVIS_VC );
	//SetMeshVisibilityMode( InsertMesh (f158,4,&offsetdirect), MESHVIS_VC );
}

    if(Lights == 0)
    {
	
	SetMeshVisibilityMode( InsertMesh (f151,1,&offset), MESHVIS_VC );
	//SetMeshVisibilityMode( InsertMesh (f152,2,&offsetalt), MESHVIS_VC );
	//SetMeshVisibilityMode( InsertMesh (f153,3,&offsetaoa), MESHVIS_VC );
	//SetMeshVisibilityMode( InsertMesh (f154,4,&offsetdirect), MESHVIS_VC );
}



}



//Ground Steering


void ShuttlePB::Steering()
 {

	double pos = GetControlSurfaceLevel(AIRCTRL_RUDDER);
	double lev = 3e6 * pos * GetAirspeed() / 10.0;

	if( oapiGetTimeAcceleration() > 10.0 )
		return;

	AddForce( _V( lev,0,0 ), _V( 0,0,0.5 ) );
	AddForce( _V( -1.0*lev,0,0 ), _V( 0,0,-0.5 ) );
	
}




// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int ShuttlePB::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

	// if (KEYMOD_SHIFT (kstate)) {

		switch (key) {
		
		case OAPI_KEY_F: // Flaps
		
		if( flaps > 0.5 )
					animactive4 = 2;
				else
					animactive4 = 1;

		//PlayVesselWave3(MySoundID,FLAPS,NOLOOP,255);
		
			
			return 1;
			
			
			case OAPI_KEY_J: // Refuel
		
		if ( refuel == 0)
		{
			refuel = 1;
                 
								
			return 1;
}
			else
     		{
											
			return 1;
}

			
			case OAPI_KEY_K: // Lights
			
	    if ( GetControlSurfaceLevel(AIRCTRL_ELEVATOR) == 0 && GetControlSurfaceLevel(AIRCTRL_AILERON) == 0  && GetControlSurfaceLevel(AIRCTRL_RUDDER) == 0 )
	    
	    {
		    
	 
		
		if( Lights == 0 )
		{
			Lights = 1;
			switched =1;
					
			return 1;
}

		if( Lights == 1 )
		{
			Lights = 0;
			switched = 1;
					
			return 1;
}
	
    
}

else return 1;
}
	return 0;
}


// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
ShuttlePB::~ShuttlePB ()
{
	
	delete pitch1;
	
}



void ShuttlePB::DefineAnimations(void)
{
	
	ANIMATIONCOMPONENT_HANDLE parent1;

	
	
   // Front Wheel
   static UINT WGrp1[1] = {84};
   
   static MGROUP_ROTATE wheel1 (0,WGrp1, 1, _V(0,-1.332981,2.818237), _V(1,0,0), (float)(360*RAD));
   
   //Rear left Wheel
   static UINT WGrp3[1] = {86};
   
   static MGROUP_ROTATE wheel2 (0,WGrp3, 1, _V(0,-1.208927,1.219066), _V(1,0,0), (float)(360*RAD));
   
   //Rear Right wheel
   static UINT WGrp4[1] = {88};
   
   static MGROUP_ROTATE wheel3 (0,WGrp4, 1, _V(0,-1.208927,1.219066), _V(1,0,0), (float)(360*RAD));
   
   anim_wheel = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheel, 0, 1, &wheel3);
   
   
   
   
  
   //Rudder
   static UINT RGrp1[7] = {68,70,75,76,77,78,80};
   
   static MGROUP_ROTATE rudderl (0,RGrp1, 7, _V(0,0.1567707,-2.988608), _V(0,1,-0.5), (float)(-18*RAD));
   
   // Cockpit Pedals Left
   static UINT RGrp2[2] = {32,34};
   
   static MGROUP_TRANSLATE rudderpl (1,RGrp2, 2, _V(0,0,-0.1));
   
   // Cockpit Pedals Right
   static UINT RGrp3[2] = {33,35};
   
   static MGROUP_TRANSLATE rudderpr (1,RGrp3, 2, _V(0,0,0.1));
   
   // Yaw Dial anims
   
   static UINT YawGrp1[1] = {1};
   
   static MGROUP_ROTATE yaw1 (4,YawGrp1, 1, _V(0,0.1,0), _V(0,0,1), (float)(25*RAD));
   
   anim_rudder = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderpl);
   AddAnimationComponent  (anim_rudder, 0, 1, &rudderpr);
   AddAnimationComponent  (anim_rudder, 0, 1, &yaw1);
   
   
   //Elevator
   static UINT EGrp1[2] = {62,65};
   
   static MGROUP_ROTATE elevator (0,EGrp1, 2, _V(0,6.041587E-02,-3.038295), _V(1,0,0), (float)(37*RAD));
   
   // Stick Forward
   static UINT EGrp2[2] = {40,41};
   
   static MGROUP_TRANSLATE elevator2 (1,EGrp2, 2,  _V(0,0,-0.2));
   
   
   anim_elevator = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator);
   AddAnimationComponent  (anim_elevator, 0, 1, &elevator2);
   
   
   
   //Left Aleron
   static UINT LAGrp1[1] = {94};
   
   static MGROUP_ROTATE laileron (0,LAGrp1, 1, _V(-2.526375,0.4686855,0.5838653), _V(1,0,-0.1), (float)(-42*RAD));
   
   
   //Steer Left L
   static UINT SLAGrp1[1] = {40};
   
   static MGROUP_ROTATE slaileron1 (1,SLAGrp1, 1, _V(-0.4106644,-5.138917E-02,0.3946063), _V(0,0,1), (float)(-65*RAD));
   
   //Steer Left R
   static UINT SLAGrp2[1] = {41};
   
   static MGROUP_ROTATE slaileron2 (1,SLAGrp2, 1, _V(0.4429169,-0.0537275,0.3946063), _V(0,0,1), (float)(-65*RAD));
   
   
   anim_laileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_laileron, 0, 1, &laileron);
   AddAnimationComponent  (anim_laileron, 0, 1, &slaileron1);
   AddAnimationComponent  (anim_laileron, 0, 1, &slaileron2);
   
   
   
   //Right Aleron
   static UINT RAGrp1[1] = {97};
   
   static MGROUP_ROTATE raileron (0,RAGrp1, 1, _V(2.526375,0.4686855,0.5838653), _V(1,0,0.1), (float)(42*RAD));
   
   //Steer Right L
   //static UINT SRAGrp1[10] = {10,11,12,13,14,15,19,20,21,23};
   
   //static MGROUP_ROTATE sraileron1 (1,SRAGrp1, 10, _V(-0.4898338,0.4320497,0.3946063), _V(0,0,1), (float)(-45*RAD));
   
   //Steer Right R
   //static UINT SRAGrp2[10] = {16,17,18,22,24,25,26,27,29,30};
   
   //static MGROUP_ROTATE sraileron2 (1,SRAGrp2, 10, _V(0.4898338,0.4320497,0.3946063), _V(0,0,1), (float)(-45*RAD));
   
   
   
   anim_raileron = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_raileron, 0, 1, &raileron);
   
   
   // Prop Anims
   
   static UINT Propa1[1] = {151};
   
   static MGROUP_ROTATE propa1 (0,Propa1, 1, _V(0,-0.2983013,3.624378), _V(0,0,1), (float)(360*RAD));
   
   static UINT Propa2[3] = {0,1,2};
   
   static MGROUP_ROTATE propa2 (2,Propa2, 3, _V(0,0,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_propelar = CreateAnimation (0);
   
   AddAnimationComponent  (anim_propelar, 0, 1, &propa1);
   AddAnimationComponent  (anim_propelar, 0, 1, &propa2);
  
   
    
 //Flaps animation
   
   //Left Flap
   static UINT FLGrp1[1] = {93};
   
   static MGROUP_ROTATE flapsl (0,FLGrp1, 1, _V(-0.5843681,0.4266355,0.589128), _V(1,0,0), (float)(-45*RAD));
   
   //Cockit Flaps Lever
   static UINT FLGrp2[1] = {4};
   
   static MGROUP_TRANSLATE flapsl2 (1,FLGrp2, 1, _V(0,-0.07,0));
   
   
   //Right Flap
   static UINT FRGrp1[1] = {99};
   
   static MGROUP_ROTATE flapsr (0,FRGrp1, 1, _V(0.5843681,0.4266355,0.589128), _V(1,0,0), (float)(-45*RAD));
   
   anim_flaps = CreateAnimation (0.5);
   
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsl2);
   AddAnimationComponent  (anim_flaps, 0, 1, &flapsr);
   
   
 
  // VSI Animations
   
   // Dial1
   static UINT dGrp0[1] = {61};
   
   static MGROUP_ROTATE dial1 (1,dGrp0, 1, _V(-0.2514253,0.118319,0), _V(0,0,1), (float)(-180*RAD));
   
   
   anim_dial1 = CreateAnimation (0.8);
   
   AddAnimationComponent  (anim_dial1, 0, 1, &dial1);
   

    // Direction Dial Animations
   
   // Dial2
   static UINT d2Grp0[1] = {29};
   
   static MGROUP_ROTATE dial2 (1,d2Grp0, 1, _V(-0.414103,0.1191555,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial2 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial2, 0, 1, &dial2);

    

   // Throttle Animation
   
   static UINT Pow0[2] = {59,60};
   
   static MGROUP_TRANSLATE powg (1,Pow0, 2, _V(0,0,0.07));
   
   anim_power = CreateAnimation (0);
   
   AddAnimationComponent  (anim_power, 0, 1, &powg);

   
    
   // Speed Dial Animations
   
   // Dial3
   static UINT d3Grp0[1] = {39};
   
   static MGROUP_ROTATE dial3 (1,d3Grp0, 1, _V(-0.5840579,0.2748638,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial3 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial3, 0, 1, &dial3);

   
   // Clock
   
   // Second Hand
   static UINT dGrp7[1] = {1};
   
   static MGROUP_ROTATE dial7 (1,dGrp7, 1, _V(-0.7193605,0.2650245,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_s = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_s, 0, 1, &dial7);
   
   // Minute Hand
   static UINT dGrp8[1] = {3};
   
   static MGROUP_ROTATE dial8 (1,dGrp8, 1, _V(-0.7193605,0.2650245,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_m = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_m, 0, 1, &dial8);
   
   // Hour Hand
   static UINT dGrp9[1] = {2};
   
   static MGROUP_ROTATE dial9 (1,dGrp9, 1, _V(-0.7193605,0.2650245,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_clock_h = CreateAnimation (0);
   
   AddAnimationComponent  (anim_clock_h, 0, 1, &dial9);

   
   // Fuel Guage Animations
   
   // Dial4
   static UINT dGrp44[1] = {6};
   
   static MGROUP_ROTATE dial44 (1,dGrp44, 1, _V(-0.7825062,0.1225076,0), _V(0,0,1), (float)(360*RAD));
   
   static UINT dGrp45[1] = {7};
   
   static MGROUP_ROTATE dial45 (1,dGrp45, 1, _V(-0.7072387,0.123645,0), _V(0,0,1), (float)(360*RAD));
   
   
   anim_dial44 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_dial44, 0, 1, &dial44);
   AddAnimationComponent  (anim_dial44, 0, 1, &dial45);

   
    // Altimeter Animations
   
   // Dial Long
   static UINT wWGrp0[1] = {10};
   
   static MGROUP_ROTATE wwheel0 (1,wWGrp0, 1, _V(-0.2561193,0.2805377,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel0 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel0, 0, 1, &wwheel0);

   // Dial Short
   static UINT wWGrp1[1] = {8};
   
   static MGROUP_ROTATE wwheel1 (1,wWGrp1, 1, _V(-0.2561193,0.2805377,0), _V(0,0,1), (float)(-360*RAD));
   
   
   anim_wheel1 = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheel1, 0, 1, &wwheel1);
   
   
   
   // Horizon Bank Guage Animations with new Hor2 Ball Pitch Anims
   
   static UINT BAGrp0[1] = {1};
   
   static MGROUP_ROTATE bank2 (3,BAGrp0, 1, _V(0,0,0), _V(0,0,1), (float)(-360*RAD));
   
   
   static UINT PAGrp1[1] = {0};
   
   pitch1 = new MGROUP_ROTATE (3,PAGrp1, 1, _V(0,0,0), _V(1,0,0), (float)(-360*RAD));
   

   
   anim_pitch = CreateAnimation (0.5);
   anim_bank = CreateAnimation (0.5);
   
   parent1 = AddAnimationComponent  (anim_bank, 0, 1, &bank2);
   AddAnimationComponent  (anim_pitch, 0, 1, pitch1, parent1);
   
   
   
   
}
	

bool ShuttlePB::clbkLoadVC (int id)
{
	
	SetCameraOffset (_V(-0.4,0.555 - move_down ,3.3 - move));
	
    SetCameraDefaultDirection (_V(0,0,1));

    SetCameraRotationRange (RAD*50, RAD*50, RAD*25, RAD*25);	
	
	
 return true;

}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ShuttlePB (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ShuttlePB*)vessel;
}
